@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *  Implementatin of DAAP (iTunes Music Sharing) GStreamer source
  *
  *  Copyright (C) 2005 Charles Schmidt <cschmidt2@emich.edu>
@@ -21,9 +22,6 @@
 
 #include "config.h"
 
-#include "rb-daap-src.h"
-#include "rb-daap-source.h"
-
 #include <string.h>
 #include <netdb.h>
 #include <netinet/in.h>
@@ -46,6 +44,10 @@
 #include <gst/base/gstpushsrc.h>
 #endif
 
+#include "rb-daap-source.h"
+#include "rb-daap-src.h"
+#include "rb-debug.h"
+
 #define RB_TYPE_DAAP_SRC (rb_daap_src_get_type())
 #define RB_DAAP_SRC(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj),RB_TYPE_DAAP_SRC,RBDAAPSrc))
 #define RB_DAAP_SRC_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass),RB_TYPE_DAAP_SRC,RBDAAPSrcClass))
@@ -660,6 +662,8 @@ rb_daap_src_open (RBDAAPSrc *src)
 		src->buffer_size = 0;
 	}
 
+	rb_debug ("Connecting to DAAP source: %s", src->daap_uri);
+
 	/* connect */
 	src->sock_fd = socket (AF_INET, SOCK_STREAM, 0);
 	if (src->sock_fd == -1) {
@@ -693,8 +697,19 @@ rb_daap_src_open (RBDAAPSrc *src)
 
 	/* construct request */
 	source = rb_daap_source_find_for_uri (src->daap_uri);
-	g_assert (source != NULL);
+	if (source == NULL) {
+		g_warning ("Unable to lookup source for URI: %s", src->daap_uri);
+		return FALSE;
+	}
+
+	/* The following can fail if the source is no longer connected */
 	headers = rb_daap_source_get_headers (source, src->daap_uri, src->seek_time, &src->seek_bytes);
+	if (headers == NULL) {
+		g_free (host);
+		g_free (path);
+		return FALSE;
+	}
+
 	request = g_strdup_printf ("GET %s HTTP/1.1\r\nHost: %s\r\n%s\r\n",
 				   path, host, headers);
 	g_free (headers);
