@@ -30,17 +30,27 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
-#include <gst/gst.h>
-#include <gst/gconf/gconf.h>
-#include <gst/play/play.h>
 #include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <nautilus-burn-recorder.h>
+#include <gst/gst.h>
+#ifdef HAVE_GSTREAMER_0_8
+#include <gst/gconf/gconf.h>
+#include <gst/play/play.h>
+#endif
 
 #include "rb-recorder.h"
-#include "rb-recorder-marshal.h"
 
 #include "rb-debug.h"
+#include "rb-marshal.h"
+
+#ifdef HAVE_GSTREAMER_0_8
+#define gst_caps_unref gst_caps_free
+#endif
+#ifndef HAVE_BURN_DRIVE_UNREF
+#define nautilus_burn_drive_unref nautilus_burn_drive_free
+#define nautilus_burn_drive_ref nautilus_burn_drive_copy
+#endif
 
 static void rb_recorder_class_init (RBRecorderClass *klass);
 static void rb_recorder_init       (RBRecorder      *recorder);
@@ -52,34 +62,38 @@ struct _RBRecorderPrivate {
         char       *tmp_dir;
         
         GstElement *pipeline;
-        
-        GstElement *src;
-        GstPad     *src_pad;
         GstElement *decoder;
+        GstElement *src;
         GstElement *typefind;
         GstElement *audioconvert;
         GstElement *audioscale;
         GstElement *encoder;
         GstElement *sink;
 
+#ifdef HAVE_GSTREAMER_0_8
+        GstPad     *src_pad;
+        GError     *error;
+
         guint       idle_id;
         guint       error_signal_id;
         guint       eos_signal_id;
+#elif HAVE_GSTREAMER_0_10
+	GstElement *capsfilter;
+	gboolean    got_audio_pad;
+#endif
+
         guint       tick_timeout_id;
 
         GTimer     *start_timer;
         guint64     start_pos;
 
         double      progress;
-
         GList      *tracks;
 
         NautilusBurnDrive    *drive;
         NautilusBurnRecorder *recorder;
 
         gboolean    playing;
-
-        GError     *error;
 };
 
 typedef enum {
@@ -93,26 +107,17 @@ typedef enum {
         LAST_SIGNAL
 } RBRecorderSignalType;
 
+#ifdef HAVE_GSTREAMER_0_8
 typedef struct {
         RBRecorder *object;
         GError     *error;
         GValue     *info;
 } RBRecorderSignal;
+#endif
 
 static guint rb_recorder_signals [LAST_SIGNAL] = { 0 };
 
-static GObjectClass *parent_class = NULL;
-
-gboolean
-rb_recorder_enabled (void)
-{
-        char    *device  = rb_recorder_get_default_device ();
-        gboolean enabled = (device != NULL);
-
-        g_free (device);
-
-        return enabled;
-}
+#define RB_RECORDER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_RECORDER, RBRecorderPrivate))
 
 GQuark
 rb_recorder_error_quark (void)
@@ -131,7 +136,6 @@ rb_recorder_class_init (RBRecorderClass *klass)
 {
         GObjectClass *object_class;
 
-        parent_class = g_type_class_peek_parent (klass);
         object_class = (GObjectClass *) klass;
 
         object_class->finalize = rb_recorder_finalize;
@@ -151,7 +155,7 @@ rb_recorder_class_init (RBRecorderClass *klass)
                               G_SIGNAL_RUN_LAST,
                               0,
                               NULL, NULL,
-                              rb_recorder_marshal_VOID__DOUBLE_LONG,
+                              rb_marshal_VOID__DOUBLE_LONG,
                               G_TYPE_NONE,
                               2,
                               G_TYPE_DOUBLE,
@@ -162,7 +166,7 @@ rb_recorder_class_init (RBRecorderClass *klass)
                               G_SIGNAL_RUN_LAST,
                               0,
                               NULL, NULL,
-                              rb_recorder_marshal_VOID__DOUBLE_LONG,
+                              rb_marshal_VOID__DOUBLE_LONG,
                               G_TYPE_NONE,
                               2,
                               G_TYPE_DOUBLE,
@@ -183,7 +187,7 @@ rb_recorder_class_init (RBRecorderClass *klass)
                               G_SIGNAL_RUN_LAST,
                               0,
                               NULL, NULL,
-                              rb_recorder_marshal_BOOLEAN__BOOLEAN_BOOLEAN_BOOLEAN,
+                              rb_marshal_BOOLEAN__BOOLEAN_BOOLEAN_BOOLEAN,
                               G_TYPE_BOOLEAN,
                               3,
                               G_TYPE_BOOLEAN,
@@ -195,7 +199,7 @@ rb_recorder_class_init (RBRecorderClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      rb_recorder_marshal_INT__VOID,
+			      rb_marshal_INT__VOID,
 			      G_TYPE_INT, 0);
         rb_recorder_signals [ERROR] =
                 g_signal_new ("error",
@@ -207,14 +211,52 @@ rb_recorder_class_init (RBRecorderClass *klass)
                               G_TYPE_NONE,
                               1,
                               G_TYPE_POINTER);
+
+        g_type_class_add_private (klass, sizeof (RBRecorderPrivate));
+}
+
+static NautilusBurnDrive *
+rb_recorder_get_default_drive (void)
+{
+        NautilusBurnDrive *drive  = NULL;
+        GList             *drives = NULL;
+
+        drives = nautilus_burn_drive_get_list (TRUE, FALSE);
+
+        if (drives)
+                drive = nautilus_burn_drive_ref ((NautilusBurnDrive*) drives->data);
+
+        g_list_foreach (drives, (GFunc)nautilus_burn_drive_unref, NULL);
+        g_list_free (drives);
+
+        return drive;
+}
+
+gboolean
+rb_recorder_enabled (void)
+{
+        NautilusBurnDrive *drive;
+        gboolean           enabled;
+
+        drive = rb_recorder_get_default_drive ();
+
+        enabled = (drive != NULL);
+
+        if (drive) {
+                nautilus_burn_drive_unref (drive);
+        }
+
+        return enabled;
 }
 
 static void
 rb_recorder_init (RBRecorder *recorder)
 {
-        recorder->priv = g_new0 (RBRecorderPrivate, 1);
+        recorder->priv = RB_RECORDER_GET_PRIVATE (recorder);
 
         recorder->priv->tmp_dir = g_strdup (g_get_tmp_dir ());
+
+        recorder->priv->drive = rb_recorder_get_default_drive ();
 }
 
 static void
@@ -234,6 +276,7 @@ rb_recorder_gst_free_pipeline (RBRecorder *recorder)
                 }
         }
 
+#ifdef HAVE_GSTREAMER_0_8
         if (recorder->priv->idle_id > 0) {
                 g_source_remove (recorder->priv->idle_id);
                 recorder->priv->idle_id = 0;
@@ -254,6 +297,11 @@ rb_recorder_gst_free_pipeline (RBRecorder *recorder)
                                                      recorder->priv->eos_signal_id);
                 recorder->priv->eos_signal_id = 0;
         }
+#elif HAVE_GSTREAMER_0_10
+	recorder->priv->got_audio_pad = FALSE;
+#endif
+
+        gst_element_set_state (recorder->priv->pipeline, GST_STATE_NULL);
 
         gst_object_unref (GST_OBJECT (recorder->priv->pipeline));
         recorder->priv->pipeline = NULL;
@@ -282,6 +330,7 @@ add_track (RBRecorder *recorder,
         return TRUE;
 }
 
+#ifdef HAVE_GSTREAMER_0_8
 static void
 eos_cb (GstElement *element,
         RBRecorder *recorder)
@@ -340,6 +389,98 @@ error_cb (GstElement      *element,
 
         rb_recorder_gst_signal_error (recorder, error->message);
 }
+#elif HAVE_GSTREAMER_0_10
+static void
+rb_recorder_gst_signal_error (RBRecorder *recorder,
+			      const char *msg)
+{
+	GError *error;
+
+	g_object_ref (recorder);
+	error = g_error_new_literal (RB_RECORDER_ERROR,
+				     RB_RECORDER_ERROR_GENERAL,
+				     msg);
+	g_signal_emit (G_OBJECT (recorder),
+		       rb_recorder_signals [ERROR],
+		       0,
+		       error);
+
+	/* close if not already closing */
+	if (recorder->priv->src_uri != NULL)
+		rb_recorder_close (recorder, NULL);
+
+	g_object_unref (recorder);
+	g_error_free (error);
+}
+
+static gboolean
+pipe_message (GstBus *bus, GstMessage *message, RBRecorder *recorder)
+{
+	switch (message->type) {
+	case GST_MESSAGE_EOS: 
+		rb_debug ("EOS");
+
+		if (recorder->priv->pipeline)
+			gst_element_set_state (recorder->priv->pipeline, GST_STATE_NULL);
+
+		g_signal_emit (G_OBJECT (recorder), 
+			       rb_recorder_signals [EOS], 0);
+		break;
+	case GST_MESSAGE_ERROR:
+	{
+		GError *error;
+		gchar *debug;
+
+		rb_debug ("Error");
+		gst_message_parse_error (message, &error, &debug);
+		if (error) {
+			rb_recorder_gst_signal_error (recorder, error->message);
+			g_error_free (error);
+		} else {
+			rb_recorder_gst_signal_error (recorder, NULL);
+		}
+
+		g_free (debug);
+		break;
+	}
+	default:
+		break;
+	}
+
+	return TRUE;
+}
+#endif
+ 
+static void
+rb_recorder_new_pad_cb (GstElement *decodebin,
+			GstPad     *pad,
+			gboolean    last,
+			RBRecorder *recorder)
+{
+	GstCaps      *caps;
+	GstStructure *str;
+	GstPad       *audio_pad;
+
+	audio_pad = gst_element_get_pad (recorder->priv->audioconvert, "sink");
+
+	/* Only link once. */
+	if (GST_PAD_IS_LINKED (audio_pad))
+		return;
+
+	/* Only link audio. */
+	caps = gst_pad_get_caps (pad);
+	str = gst_caps_get_structure (caps, 0);
+	if (!g_strrstr (gst_structure_get_name (str), "audio")) {
+		gst_caps_unref (caps);
+		return;
+	}
+
+	gst_caps_unref (caps);
+#ifdef HAVE_GSTREAMER_0_10
+	if (gst_pad_link (pad, audio_pad))
+		recorder->priv->got_audio_pad = TRUE;
+#endif
+}
 
 static void
 rb_recorder_construct (RBRecorder *recorder,
@@ -348,6 +489,9 @@ rb_recorder_construct (RBRecorder *recorder,
 {
         char    *element_name = NULL;
         GstCaps *filtercaps;
+#ifdef HAVE_GSTREAMER_0_10
+	GstBus  *bus;
+#endif
 
 #define MAKE_ELEMENT_OR_LOSE(NAME, NICE) G_STMT_START {                 \
                 element_name = #NAME ;                                  \
@@ -360,7 +504,7 @@ rb_recorder_construct (RBRecorder *recorder,
         rb_recorder_gst_free_pipeline (recorder);
 
         /* The recording pipeline looks like:
-         *  { src ! spider ! audioconvert ! audioscale
+         *  { src ! decodebin ! audioconvert ! audioscale
          *    ! audio/x-raw-int,rate=44100,width=16,depth=16 ! wavenc ! sink }
          */
 
@@ -374,11 +518,17 @@ rb_recorder_construct (RBRecorder *recorder,
                 return;
         }
 
+#ifdef HAVE_GSTREAMER_0_8
         recorder->priv->error_signal_id =
                 g_signal_connect_object (G_OBJECT (recorder->priv->pipeline),
                                          "error",
                                          G_CALLBACK (error_cb),
                                          recorder, 0);
+#elif HAVE_GSTREAMER_0_10
+	bus = gst_element_get_bus (recorder->priv->pipeline);
+	gst_bus_add_watch (bus, (GstBusFunc) pipe_message, recorder);
+	gst_object_unref (bus);
+#endif
 
         /* Construct elements */
 
@@ -387,53 +537,66 @@ rb_recorder_construct (RBRecorder *recorder,
         MAKE_ELEMENT_OR_LOSE(gnomevfssrc, src);
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->src);
 
+#ifdef HAVE_GSTREAMER_0_8
         recorder->priv->src_pad = gst_element_get_pad (recorder->priv->src, "src");
-        g_assert (recorder->priv->src_pad); /* TODO: GError */
+	if (recorder->priv->src_pad == NULL) {
+                g_set_error (error,
+                             RB_RECORDER_ERROR,
+                             RB_RECORDER_ERROR_INTERNAL,
+                             _("Could not access source pad"));
+		return;
+	}
+#endif
 
         /* The queue */
 
         MAKE_ELEMENT_OR_LOSE(typefind, typefind);
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->typefind);
 
-        MAKE_ELEMENT_OR_LOSE(spider, decoder);
+        MAKE_ELEMENT_OR_LOSE(decodebin, decoder);
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->decoder);
+	
+	g_signal_connect_object (G_OBJECT (recorder->priv->decoder), 
+				 "new_decoded_pad", G_CALLBACK (rb_recorder_new_pad_cb), 
+				 recorder, 0);
 
         MAKE_ELEMENT_OR_LOSE(audioconvert, audioconvert);
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->audioconvert);
 
+#ifdef HAVE_GSTREAMER_0_8
         MAKE_ELEMENT_OR_LOSE(audioscale, audioscale);
+#elif HAVE_GSTREAMER_0_10
+        MAKE_ELEMENT_OR_LOSE(audioresample, audioscale);
+#endif
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->audioscale);
 
+#ifdef HAVE_GSTREAMER_0_10
+	MAKE_ELEMENT_OR_LOSE(capsfilter, capsfilter);
+	gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->capsfilter);
+#endif
+	
         MAKE_ELEMENT_OR_LOSE(wavenc, encoder);
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->encoder);
 
         /* Output sink */
 
         MAKE_ELEMENT_OR_LOSE(gnomevfssink, sink);
-        if (recorder->priv->sink == NULL) {
-                g_set_error (error,
-                             RB_RECORDER_ERROR,
-                             RB_RECORDER_ERROR_NO_AUDIO,
-                             _("Could not create audio output element; check your settings"));
-                gst_object_unref (GST_OBJECT (recorder->priv->pipeline));
-                recorder->priv->pipeline = NULL;
-                return;
-        }
         gst_bin_add (GST_BIN (recorder->priv->pipeline), recorder->priv->sink);
 
-        gst_element_link_many (recorder->priv->src,
-                               recorder->priv->typefind,
-                               recorder->priv->decoder,
-                               recorder->priv->audioconvert,
-                               recorder->priv->audioscale,
-                               NULL);
-
         filtercaps = gst_caps_new_simple ("audio/x-raw-int",
                                           "channels", G_TYPE_INT, 2,
                                           "rate",     G_TYPE_INT, 44100,
                                           "width",    G_TYPE_INT, 16,
                                           "depth",    G_TYPE_INT, 16,
                                           NULL);
+#ifdef HAVE_GSTREAMER_0_8
+        gst_element_link_many (recorder->priv->src,
+                               recorder->priv->typefind,
+                               recorder->priv->decoder,
+                               recorder->priv->audioconvert,
+                               recorder->priv->audioscale,
+                               NULL);
+
         gst_element_link_filtered (recorder->priv->audioscale,
                                    recorder->priv->encoder,
                                    filtercaps);
@@ -445,6 +608,21 @@ rb_recorder_construct (RBRecorder *recorder,
         recorder->priv->eos_signal_id =
                 g_signal_connect_object (G_OBJECT (recorder->priv->pipeline), "eos",
                                          G_CALLBACK (eos_cb), recorder, 0);
+#elif HAVE_GSTREAMER_0_10
+	g_object_set (recorder->priv->capsfilter, "filter-caps",  filtercaps, NULL);
+
+	gst_element_link_many (recorder->priv->src,
+			       recorder->priv->typefind,
+			       recorder->priv->decoder,
+			       NULL);
+
+	gst_element_link_many (recorder->priv->audioconvert,
+			       recorder->priv->audioscale,
+			       recorder->priv->capsfilter,
+			       recorder->priv->encoder,
+			       recorder->priv->sink,
+			       NULL);
+#endif
 
         rb_debug ("Pipeline construction complete");
         return;
@@ -500,18 +678,16 @@ rb_recorder_finalize (GObject *object)
                         NULL);
         g_list_free (recorder->priv->tracks);
 
-        g_free (recorder->priv);
-
-        G_OBJECT_CLASS (parent_class)->finalize (object);
+        G_OBJECT_CLASS (rb_recorder_parent_class)->finalize (object);
 }
 
 RBRecorder *
 rb_recorder_new (GError **error)
 {
         RBRecorder *recorder;
-        GstElement *dummy;
 
-        rb_debug ("New rb_recorder");
+#ifdef HAVE_GSTREAMER_0_8
+        GstElement *dummy;
 
         dummy = gst_element_factory_make ("fakesink", "fakesink");
         if (!dummy
@@ -521,6 +697,7 @@ rb_recorder_new (GError **error)
                              _("Couldn't initialize scheduler.  Did you run gst-register?"));
                 return NULL;
         }
+#endif
 
         recorder = g_object_new (RB_TYPE_RECORDER, NULL, NULL);
 
@@ -530,19 +707,30 @@ rb_recorder_new (GError **error)
 static gboolean
 tick_timeout_cb (RBRecorder *recorder)
 {
-        gint64           position, total;
-        double           fraction;
-        double           rate;
-        double           elapsed;
-        double           secs;
-        static GstFormat format = GST_FORMAT_BYTES;
+        gint64 position, total;
+        double fraction;
+        double rate;
+        double elapsed;
+        double secs;
+        GstFormat format = GST_FORMAT_BYTES;
+#ifdef HAVE_GSTREAMER_0_10
+	GstState  state;
+#endif
 
         g_return_val_if_fail (recorder != NULL, FALSE);
         g_return_val_if_fail (RB_IS_RECORDER (recorder), FALSE);
         g_return_val_if_fail (recorder->priv != NULL, FALSE);
         g_return_val_if_fail (recorder->priv->pipeline != NULL, FALSE);
 
+#ifdef HAVE_GSTREAMER_0_8
         if (gst_element_get_state (recorder->priv->pipeline) != GST_STATE_PLAYING) {
+#elif HAVE_GSTREAMER_0_10
+	if (!gst_element_get_state (recorder->priv->pipeline, &state,  NULL, 3 * GST_SECOND)) {
+		g_warning (_("Could not retrieve state from processing pipeline"));
+		return TRUE;
+	}
+	if (state != GST_STATE_PLAYING) {		
+#endif
                 recorder->priv->tick_timeout_id = 0;
                 if (recorder->priv->start_timer) {
                         g_timer_destroy (recorder->priv->start_timer);
@@ -551,12 +739,16 @@ tick_timeout_cb (RBRecorder *recorder)
                 return FALSE;
         }
 
+#ifdef  HAVE_GSTREAMER_0_8
         if (!gst_pad_query (recorder->priv->src_pad, GST_QUERY_POSITION, &format, &position)) {
                 g_warning (_("Could not get current track position"));
                 return TRUE;
         }
 
         if (!gst_pad_query (recorder->priv->src_pad, GST_QUERY_TOTAL, &format, &total)) {
+#elif HAVE_GSTREAMER_0_10
+		if (!gst_element_query_position (recorder->priv->src, &format, &position) || !gst_element_query_duration (recorder->priv->src, &format, &total)) {
+#endif
                 g_warning (_("Could not get current track position"));
                 return TRUE;
         }
@@ -585,8 +777,10 @@ tick_timeout_cb (RBRecorder *recorder)
                                fraction, (long)secs);
         }
 
+#ifdef HAVE_GSTREAMER_0_8
         /* Extra kick in the pants to keep things moving on a busy system */
         gst_bin_iterate (GST_BIN (recorder->priv->pipeline));
+#endif
 
         return TRUE;
 }
@@ -603,33 +797,46 @@ rb_recorder_sync_pipeline (RBRecorder *recorder,
         rb_debug ("Syncing pipeline");
         if (recorder->priv->playing) {
                 rb_debug ("Playing pipeline");
-                if (gst_element_set_state (recorder->priv->pipeline,
-                                           GST_STATE_PLAYING) == GST_STATE_FAILURE) {
+#ifdef HAVE_GSTREAMER_0_8
+                if (gst_element_set_state (recorder->priv->pipeline, GST_STATE_PLAYING) == GST_STATE_FAILURE) {
+#elif HAVE_GSTREAMER_0_10
+                if (gst_element_set_state (recorder->priv->pipeline, GST_STATE_PLAYING) == GST_STATE_CHANGE_FAILURE) {
+#endif
                         g_set_error (error,
                                      RB_RECORDER_ERROR,
                                      RB_RECORDER_ERROR_GENERAL,
                                      _("Could not start pipeline playing"));
                         return FALSE;
-                }                               
+                }
+#ifdef HAVE_GSTREAMER_0_8
                 recorder->priv->idle_id = g_idle_add ((GSourceFunc)gst_bin_iterate,
                                                       GST_BIN (recorder->priv->pipeline));
                 recorder->priv->tick_timeout_id = g_timeout_add (200,
                                                                  (GSourceFunc)tick_timeout_cb,
                                                                  recorder);
+#elif HAVE_GSTREAMER_0_10
+		recorder->priv->tick_timeout_id = 
+			g_timeout_add (200, (GSourceFunc)tick_timeout_cb, recorder);
+#endif
         } else {
                 rb_debug ("Pausing pipeline");
-                if (gst_element_set_state (recorder->priv->pipeline,
-                                           GST_STATE_PAUSED) == GST_STATE_FAILURE) {
+#ifdef HAVE_GSTREAMER_0_8
+                if (gst_element_set_state (recorder->priv->pipeline, GST_STATE_PAUSED) == GST_STATE_FAILURE) {
+#elif HAVE_GSTREAMER_0_10
+                if (gst_element_set_state (recorder->priv->pipeline, GST_STATE_PAUSED) == GST_STATE_CHANGE_FAILURE) {
+#endif
                         g_set_error (error,
                                      RB_RECORDER_ERROR,
                                      RB_RECORDER_ERROR_GENERAL,
                                      _("Could not pause playback"));
                         return FALSE;
                 }
+#ifdef HAVE_GSTREAMER_0_8
                 if (recorder->priv->idle_id > 0) {
                         g_source_remove (recorder->priv->idle_id);
                         recorder->priv->idle_id = 0;
                 }
+#endif
                 if (recorder->priv->tick_timeout_id > 0) {
                         g_source_remove (recorder->priv->tick_timeout_id);
                         recorder->priv->tick_timeout_id = 0;
@@ -651,8 +858,10 @@ rb_recorder_close (RBRecorder *recorder,
 
         rb_debug ("Closing rb_recorder");
 
+#ifdef HAVE_GSTREAMER_0_10
+	recorder->priv->got_audio_pad = FALSE;
+#endif
         recorder->priv->playing = FALSE;
-
         g_free (recorder->priv->src_uri);
         recorder->priv->src_uri = NULL;
 
@@ -726,10 +935,14 @@ rb_recorder_open (RBRecorder   *recorder,
         if (error && *error)
                 return;
                 
+#ifdef HAVE_GSTREAMER_0_8
         if (recorder->priv->idle_id > 0) {
                 g_source_remove (recorder->priv->idle_id);
                 recorder->priv->idle_id = 0;
         }
+#elif HAVE_GSTREAMER_0_10
+	recorder->priv->got_audio_pad = FALSE;
+#endif
 
         g_object_set (G_OBJECT (recorder->priv->src), "iradio-mode", FALSE, NULL);
         gst_element_set_state (recorder->priv->src, GST_STATE_NULL);
@@ -810,7 +1023,8 @@ rb_recorder_get_device (RBRecorder  *recorder,
         drive = recorder->priv->drive;
 
         if (! drive) {
-                g_set_error (error, RB_RECORDER_ERROR,
+                g_set_error (error,
+                             RB_RECORDER_ERROR,
                              RB_RECORDER_ERROR_GENERAL,
                              _("Cannot find drive"));
                 return NULL;
@@ -834,20 +1048,28 @@ rb_recorder_set_device (RBRecorder  *recorder,
         if (error)
                 *error = NULL;
 
+        if (recorder->priv->drive) {
+                nautilus_burn_drive_unref (recorder->priv->drive);
+                recorder->priv->drive = NULL;
+        }
+
         drives = nautilus_burn_drive_get_list (TRUE, FALSE);
 
         for (tmp = drives; tmp != NULL; tmp = tmp->next) {
-                NautilusBurnDrive *drive = (NautilusBurnDrive*) tmp->data;
+                NautilusBurnDrive *drive = (NautilusBurnDrive *) tmp->data;
+
                 if (strcmp (drive->device, device) == 0) {
                         recorder->priv->drive = drive;
                         break;
                 }
-                nautilus_burn_drive_free (drive);
+
+                nautilus_burn_drive_unref (drive);
         }
         g_list_free (drives);
 
         if (! recorder->priv->drive) {
-                g_set_error (error, RB_RECORDER_ERROR,
+                g_set_error (error,
+                             RB_RECORDER_ERROR,
                              RB_RECORDER_ERROR_GENERAL,
                              _("Cannot find drive %s"),
                              device);
@@ -855,7 +1077,8 @@ rb_recorder_set_device (RBRecorder  *recorder,
         }
 
         if (! (recorder->priv->drive->type & NAUTILUS_BURN_DRIVE_TYPE_CD_RECORDER)) {
-                g_set_error (error, RB_RECORDER_ERROR,
+                g_set_error (error,
+                             RB_RECORDER_ERROR,
                              RB_RECORDER_ERROR_GENERAL,
                              _("Drive %s is not a recorder"),
                              device);
@@ -992,23 +1215,6 @@ rb_recorder_warn_data_loss_cb (NautilusBurnRecorder *cdrecorder,
         return ret;
 }
 
-static NautilusBurnDrive *
-rb_recorder_get_default_drive (void)
-{
-        NautilusBurnDrive *drive  = NULL;
-        GList             *drives = NULL;
-
-        drives = nautilus_burn_drive_get_list (TRUE, FALSE);
-
-        if (drives)
-                drive = nautilus_burn_drive_copy ((NautilusBurnDrive*) drives->data);
-
-        g_list_foreach (drives, (GFunc)nautilus_burn_drive_free, NULL);
-        g_list_free (drives);
-
-        return drive;
-}
-
 char *
 rb_recorder_get_default_device (void)
 {
@@ -1019,7 +1225,7 @@ rb_recorder_get_default_device (void)
 
         if (drive) {
                 device = g_strdup (drive->device);
-                nautilus_burn_drive_free (drive);
+                nautilus_burn_drive_unref (drive);
         }
 
         return device;
@@ -1029,20 +1235,23 @@ gint64
 rb_recorder_get_media_length (RBRecorder *recorder,
                               GError    **error)
 {
-        char  *device;
-        gint64 size;
-        gint64 secs;
+        gint64                size;
+        gint64                secs;
 
-        g_return_val_if_fail (recorder != NULL, FALSE);
-        g_return_val_if_fail (RB_IS_RECORDER (recorder), FALSE);
-        g_return_val_if_fail (recorder->priv != NULL, FALSE);
+        g_return_val_if_fail (recorder != NULL, -1);
+        g_return_val_if_fail (RB_IS_RECORDER (recorder), -1);
+        g_return_val_if_fail (recorder->priv != NULL, -1);
 
-        if (recorder->priv->drive)
-                device = g_strdup (recorder->priv->drive->device);
-        else
-                device = rb_recorder_get_default_device ();
-                
-        size = nautilus_burn_drive_get_media_size_from_path (device);
+        if (! recorder->priv->drive) {
+                g_set_error (error,
+                             RB_RECORDER_ERROR,
+                             RB_RECORDER_ERROR_INTERNAL,
+                             _("No writable drives found."));
+
+                return -1;
+        }
+
+        size = nautilus_burn_drive_get_media_size (recorder->priv->drive);
         if (size > 0) {
 #ifdef NAUTILUS_BURN_DRIVE_SIZE_TO_TIME
                 secs = NAUTILUS_BURN_DRIVE_SIZE_TO_TIME (size);
@@ -1053,14 +1262,19 @@ rb_recorder_get_media_length (RBRecorder *recorder,
                 secs = size;
         }
 
-        g_free (device);
-
         return secs;
 }
 
+#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
+#define GNUC_PACKED                             \
+  __attribute__((packed))
+#else   /* !__GNUC__ */
+#pragma pack(2)
+#endif
+
 /* Copyright (C) Bastien Nocera */
 /* From xine-lib, whoop */
-typedef struct __attribute__((__packed__)) {
+typedef struct GNUC_PACKED {
 	gint16   wFormatTag;
 	gint16   nChannels;
 	gint32   nSamplesPerSec;
@@ -1069,7 +1283,8 @@ typedef struct __attribute__((__packed__)) {
 	gint16   wBitsPerSample;
 	gint16   cbSize;
 } waveformat;
-#ifndef ATTRIBUTE_PACKED
+
+#ifndef GNUC_PACKED
 #pragma pack()
 #endif
 
@@ -1203,7 +1418,6 @@ rb_recorder_burn (RBRecorder *recorder,
         int                            res;
         gboolean                       result;
         gint64                         tracks_length;
-        gint64                         media_length;
 
         g_return_val_if_fail (recorder != NULL, RB_RECORDER_RESULT_ERROR);
         g_return_val_if_fail (RB_IS_RECORDER (recorder), RB_RECORDER_RESULT_ERROR);
@@ -1214,18 +1428,12 @@ rb_recorder_burn (RBRecorder *recorder,
         if (!recorder->priv->tracks)
                 return RB_RECORDER_RESULT_ERROR;
 
-        if (!recorder->priv->drive) {
-                char *default_device = rb_recorder_get_default_device ();
-
-                if (!default_device) {
-                        g_warning (_("Could not determine default writer device"));
-                        return RB_RECORDER_RESULT_ERROR;
-                }
-
-                rb_recorder_set_device  (recorder, default_device, error);
-                g_free (default_device);
-                if (error && *error)
-                        return RB_RECORDER_RESULT_ERROR;
+        if (! recorder->priv->drive) {
+                g_set_error (error,
+                             RB_RECORDER_ERROR,
+                             RB_RECORDER_ERROR_INTERNAL,
+                             _("No writable drives found."));
+                return RB_RECORDER_RESULT_ERROR;
         }
 
         tracks_length = get_tracks_length (recorder, error);
@@ -1237,28 +1445,6 @@ rb_recorder_burn (RBRecorder *recorder,
                 return RB_RECORDER_RESULT_ERROR;
         }
 
-        media_length = rb_recorder_get_media_length (recorder, error);
-
-        /* don't fail here if media length cannot be determined
-         * nautilus_burn_recorder_write_tracks will fail and issue a signal */
-        if ((media_length > 0) && (tracks_length > media_length)) {
-                char *duration_string = g_strdup_printf ("%" G_GINT64_FORMAT, tracks_length / 60);
-                char *media_duration_string = g_strdup_printf ("%" G_GINT64_FORMAT, media_length / 60);
-
-                g_set_error (error,
-                             RB_RECORDER_ERROR,
-                             RB_RECORDER_ERROR_GENERAL,
-                             _("This playlist is %s minutes long.  "
-                               "This exceeds the %s minute length of the media in the drive."),
-                             duration_string,
-                             media_duration_string);
-
-                g_free (duration_string);
-                g_free (media_duration_string);
-
-                return RB_RECORDER_RESULT_ERROR;
-        }
-
         cdrecorder = nautilus_burn_recorder_new ();
         recorder->priv->recorder = cdrecorder;
 
