@@ -40,8 +40,8 @@
 #include <libxml/SAX.h>
 #include <libxml/parserInternals.h>
 
+#include "rhythmdb-private.h"
 #include "rhythmdb-tree.h"
-#include "rhythmdb-query-model.h"
 #include "rhythmdb-property-model.h"
 #include "rb-debug.h"
 #include "rb-util.h"
@@ -74,7 +74,8 @@ static void rhythmdb_tree_entry_delete_by_type (RhythmDB *adb, RhythmDBEntryType
 static RhythmDBEntry * rhythmdb_tree_entry_lookup_by_location (RhythmDB *db, const char *uri);
 static void rhythmdb_tree_entry_foreach (RhythmDB *adb, GFunc func, gpointer user_data);
 static void rhythmdb_tree_do_full_query (RhythmDB *db, GPtrArray *query,
-					 GtkTreeModel *main_model, gboolean *cancel);
+					 RhythmDBQueryResults *results,
+					 gboolean *cancel);
 static gboolean rhythmdb_tree_evaluate_query (RhythmDB *adb, GPtrArray *query,
 				       RhythmDBEntry *aentry);
 
@@ -94,7 +95,7 @@ static void rhythmdb_hash_tree_foreach (RhythmDB *adb,
 					gpointer data);
 
 
-#define RHYTHMDB_TREE_XML_VERSION "1.0"
+#define RHYTHMDB_TREE_XML_VERSION "1.2"
 
 static void destroy_tree_property (RhythmDBTreeProperty *prop);
 static RhythmDBTreeProperty *get_or_create_album (RhythmDBTree *db, RhythmDBTreeProperty *artist,
@@ -209,7 +210,9 @@ struct RhythmDBTreeLoadContext
 	GString *buf;
 	RhythmDBPropType propid;
 
+	/* updating */
 	gboolean has_date;
+	gboolean canonicalise_uris;
 };
 
 static void
@@ -228,10 +231,30 @@ rhythmdb_tree_parser_start_element (struct RhythmDBTreeLoadContext *ctx,
 	{
 	case RHYTHMDB_TREE_PARSER_STATE_START:
 	{
-		if (!strcmp (name, "rhythmdb"))
+		if (!strcmp (name, "rhythmdb")) {
 			ctx->state = RHYTHMDB_TREE_PARSER_STATE_RHYTHMDB;
-		else
+			for (; *attrs; attrs +=2) {
+				if (!strcmp (*attrs, "version")) {
+					const char *version = *(attrs+1);
+
+					if (!strcmp (version, "1.0") || !strcmp (version, "1.1")) {
+						ctx->canonicalise_uris = TRUE;
+						rb_debug ("old version of rhythmdb, performing URI canonicalisation for all entries");
+					} else if (!strcmp (version, "1.2")) {
+						/* current version*/
+					} else {
+						/* too new */
+						g_assert_not_reached ();
+					}
+				} else {
+					g_assert_not_reached ();
+				}
+			}
+
+		} else {
 			ctx->in_unknown_elt = TRUE;
+		}
+
 		break;
 	}
 	case RHYTHMDB_TREE_PARSER_STATE_RHYTHMDB:
@@ -250,6 +273,8 @@ rhythmdb_tree_parser_start_element (struct RhythmDBTreeLoadContext *ctx,
 						type = RHYTHMDB_ENTRY_TYPE_PODCAST_POST;
 					else if (!strcmp (typename, "podcast-feed"))
 						type = RHYTHMDB_ENTRY_TYPE_PODCAST_FEED;
+					else if (!strcmp (typename, "ignore"))
+						type = RHYTHMDB_ENTRY_TYPE_IGNORE;
 					else
 						return;
 					type_set = TRUE;
@@ -295,6 +320,12 @@ parse_ulong (const char *buffer)
 		return MIN (val, G_MAXUINT32);
 }
 
+static gboolean
+parse_boolean (const char *buffer)
+{
+	return (parse_ulong (buffer) > 0);
+}
+
 static void
 rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx, const char *name)
 {
@@ -332,9 +363,33 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx, const cha
 		}
 		
 		if (ctx->entry->location != NULL) {
-			rhythmdb_tree_entry_new (RHYTHMDB (ctx->db), ctx->entry);
-			rhythmdb_entry_insert (RHYTHMDB (ctx->db), ctx->entry);
-			rhythmdb_commit (RHYTHMDB (ctx->db));
+			RhythmDBEntry *entry;
+
+			entry = g_hash_table_lookup (ctx->db->priv->entries, ctx->entry->location);
+			if (entry == NULL) {
+				rhythmdb_tree_entry_new (RHYTHMDB (ctx->db), ctx->entry);
+				rhythmdb_entry_insert (RHYTHMDB (ctx->db), ctx->entry);
+				rhythmdb_commit (RHYTHMDB (ctx->db));
+			} else {
+				rb_debug ("found entry with duplicate location %s. merging metadata", ctx->entry->location);
+				entry->play_count += ctx->entry->play_count;
+
+				if (entry->rating < 0.01)
+					entry->rating = ctx->entry->rating;
+				else if (ctx->entry->rating > 0.01)
+					entry->rating = (entry->rating + ctx->entry->rating) / 2;
+
+				if (ctx->entry->last_played > entry->last_played)
+					entry->last_played = ctx->entry->last_played;
+
+				if (ctx->entry->first_seen < entry->first_seen)
+					entry->first_seen = ctx->entry->first_seen;
+
+				if (ctx->entry->last_seen > entry->last_seen)
+					entry->last_seen = ctx->entry->last_seen;
+				
+				rhythmdb_entry_unref (RHYTHMDB (ctx->db), ctx->entry);
+			}
 		} else {
 			rb_debug ("found entry without location");
 			rhythmdb_entry_unref (RHYTHMDB (ctx->db), ctx->entry);
@@ -386,7 +441,10 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx, const cha
 			ctx->entry->file_size = parse_ulong (ctx->buf->str);
 			break;
 		case RHYTHMDB_PROP_LOCATION:
-			ctx->entry->location = g_strdup (ctx->buf->str);
+			if (ctx->canonicalise_uris)
+				ctx->entry->location = rb_canonicalise_uri (ctx->buf->str);
+			else
+				ctx->entry->location = g_strdup (ctx->buf->str);
 			break;
 		case RHYTHMDB_PROP_MOUNTPOINT:
 			/* remove this from old podcast-post entries */
@@ -429,6 +487,9 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx, const cha
 		case RHYTHMDB_PROP_MIMETYPE:
 			ctx->entry->mimetype = rb_refstring_new (ctx->buf->str);
 			break;
+		case RHYTHMDB_PROP_HIDDEN:
+			ctx->entry->hidden = parse_boolean (ctx->buf->str);
+			break;
 		case RHYTHMDB_PROP_STATUS:
 			ctx->entry->podcast->status = parse_ulong (ctx->buf->str);
 			break;			
@@ -462,10 +523,11 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx, const cha
 		case RHYTHMDB_PROP_ARTIST_FOLDED:
 		case RHYTHMDB_PROP_ALBUM_FOLDED:
 		case RHYTHMDB_PROP_LAST_PLAYED_STR:
-		case RHYTHMDB_PROP_HIDDEN:
 		case RHYTHMDB_PROP_PLAYBACK_ERROR:
 		case RHYTHMDB_PROP_FIRST_SEEN_STR:
+		case RHYTHMDB_PROP_LAST_SEEN_STR:
 		case RHYTHMDB_PROP_SEARCH_MATCH:
+		case RHYTHMDB_PROP_YEAR:
 		case RHYTHMDB_NUM_PROPERTIES:
 			g_assert_not_reached ();
 			break;
@@ -630,6 +692,13 @@ save_entry_ulong (struct RhythmDBTreeSaveContext *ctx,
 	write_elt_name_close (ctx, elt_name);
 }
 
+static void
+save_entry_boolean (struct RhythmDBTreeSaveContext *ctx,
+		    const xmlChar *elt_name, gboolean val)
+{
+	save_entry_ulong (ctx, elt_name, val ? 1 : 0, FALSE);
+}
+
 static void
 save_entry_uint64 (struct RhythmDBTreeSaveContext *ctx, const xmlChar *elt_name,
 		   guint64 num)
@@ -681,8 +750,11 @@ save_entry (RhythmDBTree *db, RhythmDBEntry *entry, struct RhythmDBTreeSaveConte
 		RHYTHMDB_FWRITE_STATICSTR ("podcast-post", ctx->handle, ctx->error);
 	} else if (entry->type == RHYTHMDB_ENTRY_TYPE_PODCAST_FEED) {
 		RHYTHMDB_FWRITE_STATICSTR ("podcast-feed", ctx->handle, ctx->error);
-	} else
+	} else if (entry->type == RHYTHMDB_ENTRY_TYPE_IGNORE) {
+		RHYTHMDB_FWRITE_STATICSTR ("ignore", ctx->handle, ctx->error);
+	} else {
 		g_assert_not_reached ();
+	}
 
 	RHYTHMDB_FWRITE_STATICSTR ("\">\n", ctx->handle, ctx->error);
 		
@@ -775,6 +847,9 @@ save_entry (RhythmDBTree *db, RhythmDBEntry *entry, struct RhythmDBTreeSaveConte
 		case RHYTHMDB_PROP_LAST_PLAYED:
 			save_entry_ulong (ctx, elt_name, entry->last_played, FALSE);
 			break;
+		case RHYTHMDB_PROP_HIDDEN:
+			save_entry_boolean (ctx, elt_name, entry->hidden);
+			break;
 		case RHYTHMDB_PROP_STATUS:
 			if (entry->podcast)
 				save_entry_ulong (ctx, elt_name, entry->podcast->status, FALSE);
@@ -816,10 +891,11 @@ save_entry (RhythmDBTree *db, RhythmDBEntry *entry, struct RhythmDBTreeSaveConte
 		case RHYTHMDB_PROP_ARTIST_FOLDED:
 		case RHYTHMDB_PROP_ALBUM_FOLDED:
 		case RHYTHMDB_PROP_LAST_PLAYED_STR:
-		case RHYTHMDB_PROP_HIDDEN:
 		case RHYTHMDB_PROP_PLAYBACK_ERROR:
 		case RHYTHMDB_PROP_FIRST_SEEN_STR:
+		case RHYTHMDB_PROP_LAST_SEEN_STR:
 		case RHYTHMDB_PROP_SEARCH_MATCH:
+		case RHYTHMDB_PROP_YEAR:
 		case RHYTHMDB_NUM_PROPERTIES:
 			break;
 		}
@@ -853,20 +929,23 @@ rhythmdb_tree_save (RhythmDB *rdb)
 	ctx.handle = f;
 	ctx.error = NULL;
 	RHYTHMDB_FWRITE_STATICSTR ("<?xml version=\"1.0\" standalone=\"yes\"?>\n"
-				   "<rhythmdb version=\"" RHYTHMDB_TREE_XML_VERSION "\">", 
+				   "<rhythmdb version=\"" RHYTHMDB_TREE_XML_VERSION "\">\n", 
 				   ctx.handle, ctx.error);
 
-	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_SONG, 
-				    (RBTreeEntryItFunc)save_entry, 
+	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_SONG,
+				    (RBTreeEntryItFunc)save_entry,
 				    NULL, NULL, NULL, &ctx);
-	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_IRADIO_STATION, 
-				    (RBTreeEntryItFunc)save_entry, 
+	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_IRADIO_STATION,
+				    (RBTreeEntryItFunc)save_entry,
 				    NULL, NULL, NULL, &ctx);
-	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_PODCAST_POST, 
-				    (RBTreeEntryItFunc)save_entry, 
+	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_PODCAST_POST,
+				    (RBTreeEntryItFunc)save_entry,
 				    NULL, NULL, NULL, &ctx);
-	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_PODCAST_FEED, 
-				    (RBTreeEntryItFunc)save_entry, 
+	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_PODCAST_FEED,
+				    (RBTreeEntryItFunc)save_entry,
+				    NULL, NULL, NULL, &ctx);
+	rhythmdb_hash_tree_foreach (rdb, RHYTHMDB_ENTRY_TYPE_IGNORE,
+				    (RBTreeEntryItFunc)save_entry,
 				    NULL, NULL, NULL, &ctx);
 
 	RHYTHMDB_FWRITE_STATICSTR ("</rhythmdb>\n", ctx.handle, ctx.error);
@@ -1447,6 +1526,24 @@ evaluate_conjunctive_subquery (RhythmDBTree *dbtree, GPtrArray *query,
 
 			break;
 		}
+		case RHYTHMDB_QUERY_PROP_PREFIX:
+		case RHYTHMDB_QUERY_PROP_SUFFIX:
+		{
+			const char *value_s;
+			const char *entry_s;
+			
+			g_assert (rhythmdb_get_property_type (db, data->propid) == G_TYPE_STRING);
+
+			value_s = g_value_get_string (data->val);
+			entry_s = rhythmdb_entry_get_string (entry, data->propid);
+
+			if (data->type == RHYTHMDB_QUERY_PROP_PREFIX && !g_str_has_prefix (entry_s, value_s))
+					return FALSE;
+			if (data->type == RHYTHMDB_QUERY_PROP_SUFFIX && !g_str_has_suffix (entry_s, value_s))
+					return FALSE;
+			
+			break;
+		}
 		case RHYTHMDB_QUERY_PROP_LIKE:
 		case RHYTHMDB_QUERY_PROP_NOT_LIKE:
 		{
@@ -1750,7 +1847,7 @@ struct RhythmDBTreeQueryGatheringData
 	RhythmDBTree *db;
 	GPtrArray *queue;
 	GHashTable *entries;
-	RhythmDBQueryModel *main_model;
+	RhythmDBQueryResults *results;
 };
 
 static void
@@ -1759,8 +1856,10 @@ do_query_recurse (RhythmDBTree *db, GPtrArray *query, RhythmDBTreeTraversalFunc
 {
 	GList *conjunctions, *tem;
 
+	if (query == NULL)
+		return;
+	
 	conjunctions = split_query_by_disjunctions (db, query);
-
 	rb_debug ("doing recursive query, %d conjunctions", g_list_length (conjunctions));
 
 	if (conjunctions == NULL)
@@ -1796,7 +1895,7 @@ handle_entry_match (RhythmDB *db, RhythmDBEntry *entry,
 
 	g_ptr_array_add (data->queue, entry);
 	if (data->queue->len > RHYTHMDB_QUERY_MODEL_SUGGESTED_UPDATE_CHUNK) {
-		rhythmdb_query_model_add_entries (data->main_model, data->queue);
+		rhythmdb_query_results_add_results (data->results, data->queue);
 		data->queue = g_ptr_array_new ();
 	}
 }
@@ -1804,18 +1903,18 @@ handle_entry_match (RhythmDB *db, RhythmDBEntry *entry,
 static void
 rhythmdb_tree_do_full_query (RhythmDB *adb,
 			     GPtrArray *query,
-			     GtkTreeModel *main_model,
+			     RhythmDBQueryResults *results,
 			     gboolean *cancel)
 {
 	RhythmDBTree *db = RHYTHMDB_TREE (adb);
 	struct RhythmDBTreeQueryGatheringData *data = g_new0 (struct RhythmDBTreeQueryGatheringData, 1);
 
-	data->main_model = RHYTHMDB_QUERY_MODEL (main_model);
+	data->results = results;
 	data->queue = g_ptr_array_new ();
 
 	do_query_recurse (db, query, (RhythmDBTreeTraversalFunc) handle_entry_match, data, cancel);
 
-	rhythmdb_query_model_add_entries (data->main_model, data->queue);
+	rhythmdb_query_results_add_results (data->results, data->queue);
 
 	g_free (data);
 }
