@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *  arch-tag: Implementation of removable media source object (based of the ipod source)
  *
  *  Copyright (C) 2004 Christophe Fergeau  <teuf@gnome.org>
@@ -22,14 +23,16 @@
 
 #include <config.h>
 
+#include <string.h>
+
+#include <glib/gi18n.h>
 #include <gtk/gtktreeview.h>
 #include <gtk/gtkicontheme.h>
 #include <gtk/gtkiconfactory.h>
-#include <string.h>
-#include "rhythmdb.h"
-#include <libgnome/gnome-i18n.h>
 #include <libgnomevfs/gnome-vfs-volume.h>
 #include <libgnomevfs/gnome-vfs-volume-monitor.h>
+
+#include "rhythmdb.h"
 #include "eel-gconf-extensions.h"
 #include "rb-removable-media-source.h"
 #include "rb-stock-icons.h"
@@ -51,7 +54,6 @@ static void rb_removable_media_source_get_property (GObject *object,
 			                  GValue *value,
 			                  GParamSpec *pspec);
 
-static GdkPixbuf *impl_get_pixbuf (RBSource *source);
 static void impl_delete_thyself (RBSource *source);
 
 
@@ -60,7 +62,7 @@ typedef struct
 	GnomeVFSVolume *volume;
 } RBRemovableMediaSourcePrivate;
 
-G_DEFINE_TYPE (RBRemovableMediaSource, rb_removable_media_source, RB_TYPE_LIBRARY_SOURCE)
+G_DEFINE_TYPE (RBRemovableMediaSource, rb_removable_media_source, RB_TYPE_BROWSER_SOURCE)
 #define REMOVABLE_MEDIA_SOURCE_GET_PRIVATE(o)   (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_REMOVABLE_MEDIA_SOURCE, RBRemovableMediaSourcePrivate))
 
 enum
@@ -76,17 +78,17 @@ rb_removable_media_source_class_init (RBRemovableMediaSourceClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
-	RBLibrarySourceClass *library_source_class = RB_LIBRARY_SOURCE_CLASS (klass);
+	RBBrowserSourceClass *browser_source_class = RB_BROWSER_SOURCE_CLASS (klass);
 
 	object_class->constructor = rb_removable_media_source_constructor;
 	object_class->dispose = rb_removable_media_source_dispose;
 	object_class->set_property = rb_removable_media_source_set_property;
 	object_class->get_property = rb_removable_media_source_get_property;
 
-	source_class->impl_get_pixbuf = impl_get_pixbuf;
 	source_class->impl_delete_thyself = impl_delete_thyself;
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
-	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_false_function;
+	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_true_function;
+	source_class->impl_can_paste = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_receive_drag = NULL;
 	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_false_function;
@@ -95,9 +97,9 @@ rb_removable_media_source_class_init (RBRemovableMediaSourceClass *klass)
 	source_class->impl_get_config_widget = NULL;
 	source_class->impl_show_popup = (RBSourceFeatureFunc) rb_false_function;
 
-	library_source_class->impl_get_paned_key = NULL;
-	library_source_class->impl_has_first_added_column = (RBLibrarySourceFeatureFunc) rb_false_function;
-	library_source_class->impl_has_drop_support = (RBLibrarySourceFeatureFunc) rb_false_function;
+	browser_source_class->impl_get_paned_key = NULL;
+	browser_source_class->impl_has_first_added_column = (RBBrowserSourceFeatureFunc) rb_false_function;
+	browser_source_class->impl_has_drop_support = (RBBrowserSourceFeatureFunc) rb_false_function;
 
 	g_object_class_install_property (object_class,
 					 PROP_VOLUME,
@@ -113,7 +115,6 @@ rb_removable_media_source_class_init (RBRemovableMediaSourceClass *klass)
 static void
 rb_removable_media_source_init (RBRemovableMediaSource *self)
 {
-
 }
 
 static GObject *
@@ -122,17 +123,38 @@ rb_removable_media_source_constructor (GType type, guint n_construct_properties,
 {
 	GObject *source; 
 	GnomeVFSVolume *volume;
+	GnomeVFSDrive *drive;
 	char *display_name;
+	gint size;
+	char *icon_name;
+	GtkIconTheme *theme;
+	GdkPixbuf *pixbuf;
 
 	source = G_OBJECT_CLASS(rb_removable_media_source_parent_class)
 			->constructor (type, n_construct_properties, construct_properties);
 
 	g_object_get (source, "volume", &volume, NULL);
-	
-	display_name = gnome_vfs_volume_get_display_name (volume);
+	drive = gnome_vfs_volume_get_drive (volume);
+	if (drive != NULL) {
+		display_name = gnome_vfs_drive_get_display_name (drive);
+		gnome_vfs_drive_unref (drive);
+	} else {
+		display_name = gnome_vfs_volume_get_display_name (volume);
+	}
 	g_object_set (source, "name", display_name, NULL);
 	g_free (display_name);
 
+	icon_name = gnome_vfs_volume_get_icon (volume);
+	theme = gtk_icon_theme_get_default ();
+	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
+	pixbuf = gtk_icon_theme_load_icon (theme, icon_name, size, 0, NULL);
+	g_free (icon_name);
+
+	rb_source_set_pixbuf (RB_SOURCE (source), pixbuf);
+	if (pixbuf != NULL) {
+		g_object_unref (pixbuf);
+	}
+
 	g_object_unref (volume);
 
 	return source;
@@ -160,8 +182,7 @@ rb_removable_media_source_set_property (GObject *object,
 {
 	RBRemovableMediaSourcePrivate *priv = REMOVABLE_MEDIA_SOURCE_GET_PRIVATE (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_VOLUME:
 		if (priv->volume) {
 			gnome_vfs_volume_unref (priv->volume);
@@ -183,8 +204,7 @@ rb_removable_media_source_get_property (GObject *object,
 {
 	RBRemovableMediaSourcePrivate *priv = REMOVABLE_MEDIA_SOURCE_GET_PRIVATE (object);
 
-	switch (prop_id)
-	{
+	switch (prop_id) {
 	case PROP_VOLUME:
 		gnome_vfs_volume_ref (priv->volume);
 		g_value_take_object (value, priv->volume);
@@ -195,21 +215,6 @@ rb_removable_media_source_get_property (GObject *object,
 	}
 }
 
-static GdkPixbuf *
-impl_get_pixbuf (RBSource *source)
-{
-	RBRemovableMediaSourcePrivate *priv = REMOVABLE_MEDIA_SOURCE_GET_PRIVATE (source);
-	gint size;
-
-	char *icon_name = gnome_vfs_volume_get_icon (priv->volume);
-	GtkIconTheme *theme = gtk_icon_theme_get_default ();
-	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
-	GdkPixbuf *icon = gtk_icon_theme_load_icon (theme, icon_name, size, 0, NULL);
-	g_free (icon_name);
-
-	return icon;
-}
-
 static void
 impl_delete_thyself (RBSource *source)
 {
